import os
main_dir = os.path.split(os.path.abspath(__file__))[0]
matriz_path=os.path.join(main_dir,"matriz.txt")
matriz = open(matriz_path,'r')

Matriz = [[]]
i = 0

for j in matriz.read():
	if(j == '\n'):
		Matriz.append([])
		i += 1
	else: Matriz[i].append(int(j))

def mat():
    newMat=[[0]*len(Matriz[0]) for i in range(len(Matriz))]
    for i in range(len(Matriz)):
        for j in range(len(Matriz[0])):
            newMat[i][j]=Matriz[i][j]
    return newMat



import pygame
import os
from pygame.locals import *
from Patoman import main as level1

main_dir = os.path.split(os.path.abspath(__file__))[0]

game_volume=50
def load_img(name):
   path=os.path.join(main_dir,"imgs",name)
   return pygame.image.load(path)

def load_sound(name):
   path=os.path.join(main_dir,"sounds",name)
   this_sound = pygame.mixer.Sound(path)
   this_sound.set_volume(0.3)
   return this_sound

def load_font(name,size):
   path=os.path.join(main_dir,"fonts",name)
   return pygame.font.Font(path,size)

def play_music(name,volume):
   path=os.path.join(main_dir,"sounds",name)
   pygame.mixer.music.load(path)
   pygame.mixer.music.set_volume(volume)
   pygame.mixer.music.play(loops=-1)


def run():
   
   pygame.init() #initializing pygame
   if not pygame.mixer.get_init():
       pygame.mixer.init()

   # Limitations (inclusive) for menu options:
   #top: TS*32 bottom: TS*66
   #left: TS*4 right: TS*50

   #Directions
   UP    = 0
   LEFT  = 1
   DOWN  = 2
   RIGHT = 3
   STOP  = 4

   #loading images
   patoFC = load_img('patoFC.png')
   patoFD = load_img('patoFD.png')
   patoFB = load_img('patoFB.png')
   patoFE = load_img('patoFE.png')
   patoAC = load_img('patoAC.png')
   patoAD = load_img('patoAD.png')
   patoAB = load_img('patoAB.png')
   patoAE = load_img('patoAE.png')
   arrow = load_img('arrow.png')
   
   font=load_font("emulogic.ttf",16)
   font_menu=load_font("emulogic.ttf",32)
   TS=8 #setting the size in pixels of a tile

   screen = pygame.display.set_mode((TS * 56, TS * 72)) #sets screen size
   pygame.display.set_caption("Pato-man") #sets screen title

   clock = pygame.time.Clock() #create clock object

   class Header():

       def __init__(self):
           self.title=load_img('patoman_logo.png'); #defines title image


       def display(self):
           #writes text to screen
           screen.blit(font.render("Controls: -Arrows to move",False,pygame.Color('White')),(16,160))
           screen.blit(font.render("          -Enter to select",False,pygame.Color('White')),(16,176))
           screen.blit(font.render("          -Esc to pause",False,pygame.Color('White')),(16,192))
           screen.blit(self.title,(81,64)) #shows title image

   class Main_Menu():

       def __init__(self):
           self.size = 3
           self.type = "Main Menu"
           self.positions=[(TS*15,TS*38+4),(TS*9,TS*44+4),(TS*12,TS*50+4),(TS*16,TS*56+4)]

       def display(self):
           screen.blit(font_menu.render("Start", False, pygame.Color('White')),(TS*19,TS*38))
           screen.blit(font_menu.render("Settings", False, pygame.Color('White')),(TS*13,TS*44))
           screen.blit(font_menu.render("Credits", False, pygame.Color('White')),(TS*15,TS*50))
           screen.blit(font_menu.render("Quit", False, pygame.Color('White')),(TS*20,TS*56))

   class Start_Menu():

       def __init__(self):
           self.size = 2
           self.type = "Start Menu"
           self.positions = [(TS*3,TS*41+4),(TS*4,TS*47+4),(TS*13,TS*53+4)]

       def display(self):
           screen.blit(font_menu.render("Normal Mode", False, pygame.Color('White')),(TS*7,TS*41))
           screen.blit(font_menu.render("Gamer Mode", False, pygame.Color('White')),(TS*8,TS*47))
           screen.blit(font_menu.render("Return", False, pygame.Color('White')),(TS*17,TS*53))

   class Settings_Menu():

       def __init__(self):
           self.size = 1
           self.type = "Settings Menu"
           self.positions = [(TS*7,TS*44+4),(TS*13,TS*50+4)]

       def display(self):
           global game_volume
           screen.blit(font_menu.render("Volume", False, pygame.Color('White')),(TS*11,TS*44))
           screen.blit(font_menu.render(str(game_volume), False, pygame.Color('White')),(300,TS*44))
           screen.blit(font_menu.render("Return", False, pygame.Color('White')),(TS*17,TS*50))

   class Credits_Menu():

       def __init__(self):
           self.size = 0
           self.type = "Credits Menu"
           self.positions = [(TS*19,490)]

       def display(self):
           screen.blit(font.render("This game was made as a", False, pygame.Color('White')),(TS*4,288))
           screen.blit(font.render("college project for UFCG", False, pygame.Color('White')),(TS*4,304))
           screen.blit(font.render("Developers:", False, pygame.Color('White')),(TS*4,336))
           screen.blit(font.render("Abraao Caiana de Freitas", False, pygame.Color('White')),(TS*4,368))
           screen.blit(font.render("Augusto Nunes Zacarias", False, pygame.Color('White')),(TS*4,400))
           screen.blit(font.render("Davi Henrique Silva", False, pygame.Color('White')),(TS*4,432))
           screen.blit(font.render("Joao Gabriel Abrantes", False, pygame.Color('White')),(TS*4,464))
         
           screen.blit(font.render("Return", False, pygame.Color('White')),(TS*23,496))

   class Cursor():
       
       def __init__(self):
           self.image = arrow
           self.index = 0
           self.direction = STOP
           self.size_menu = 3

       def move(self):
           global game_volume
           self.size_menu=menu.size
           if pygame.mixer.get_init():
               pygame.mixer.stop()
           if self.direction == RIGHT:
              if menu.type=="Settings Menu" and self.pos()==menu.positions[0]:
                 if game_volume<100:
                    game_volume+=10
           elif self.direction == LEFT:
              if menu.type=="Settings Menu" and self.pos()==menu.positions[0]:
                 if game_volume>0:
                    game_volume-=10
           elif self.direction == DOWN:
               if self.index == self.size_menu:
                   self.index = 0
               else:
                   self.index += 1
           elif self.direction == UP:
               if self.index == 0:
                   self.index = self.size_menu
               else:
                   self.index -= 1

           self.direction = STOP
       def pos(self):
           return menu.positions[self.index]

       def display(self):
           screen.blit(self.image,self.pos())
       
   class Animation():

       def __init__(self):
           self.pos=(TS*1,TS*29) #starting postion
           self.img=patoAD #starting image
           self.aberto=True #Variable to make mouth open and close

       def animate(self):
           self.aberto=not self.aberto 
           screen.blit(self.img,self.pos)
           if self.pos[1]==TS*67 and TS*2<=self.pos[0]<=TS*52:
               self.img=patoAE if self.aberto else patoFE
               self.pos=(self.pos[0]-TS,self.pos[1]) #moves left

           elif self.pos[0]==TS*52 and TS*66>=self.pos[1]>=TS*29:
               self.img=patoAB if self.aberto else patoFB
               self.pos=(self.pos[0],self.pos[1]+TS) #moves down

           elif self.pos[1]==TS*29 and TS*51>=self.pos[0]>=TS*1:
               self.img=patoAD if self.aberto else patoFD
               self.pos=(self.pos[0]+TS,self.pos[1]) #moves right

           elif self.pos[0]==TS*1 and TS*68>=self.pos[1]>=TS*30:
               self.img=patoAC if self.aberto else patoFC
               self.pos=(self.pos[0],self.pos[1]-TS) #moves up

   #creating classes        
   header=Header()
   animation=Animation()
   menu=Main_Menu()
   cursor=Cursor()

   main_menu = True
   start_menu = False
   settings_menu = False
   credits_menu = False
   play_music('menu_music.wav',game_volume/200)
   while True:
       clock.tick(15)

       for event in pygame.event.get():

               if event.type == pygame.QUIT:
                   pygame.quit() #Quit game

               #Directions keys
               if event.type == KEYDOWN:
                   if event.key == K_UP:
                       cursor.direction = UP
                   if event.key == K_DOWN:
                       cursor.direction = DOWN
                   if event.key == K_LEFT:
                       cursor.direction = LEFT
                   if event.key == K_RIGHT:
                       cursor.direction = RIGHT
                   
                   if event.key == K_RETURN:
                       if menu.type == "Main Menu":
                           if cursor.index == 0:
                               menu = Start_Menu()
                           elif cursor.index == 1:
                               menu = Settings_Menu()
                           elif cursor.index == 2:
                               menu = Credits_Menu()
                           elif cursor.index == 3:
                               pygame.quit()
                           cursor.index = 0 #initialize cursor on the top of new menu

                       elif menu.type == "Start Menu":
                           if cursor.pos() == menu.positions[0]:   #start level 1
                               level1.setup(game_volume/200,False)
                           elif cursor.pos() == menu.positions[1]: #start level 1
                               level1.setup(game_volume/200,True)
                           elif cursor.pos() == menu.positions[2]: #return buttom
                               menu = Main_Menu()
                           cursor.index = 0 #reinitialize cursor in the main menu on the right position

                       elif menu.type == "Settings Menu":
                           if cursor.pos() == menu.positions[0]: #already handled in cursor.move
                               pass
                           if cursor.pos() == menu.positions[1]: #return buttom
                               menu = Main_Menu()
                           cursor.index = 1 #reinitialize cursor in the main menu on the right position

                       elif menu.type == "Credits Menu":
                           if cursor.pos() == menu.positions[0]: #return buttom
                               menu = Main_Menu()
                           cursor.index = 2 #reinitialize cursor in the main menu on the right position

       screen.fill((0, 0, 0)) #Display objects on screen
       header.display() #Displays header
       animation.animate() #Shows duck animation
       cursor.move() #Moves cursor
       cursor.display() #Displays cursor
       pygame.mixer.music.set_volume(game_volume/200)#update menu song volume

       if main_menu == True:
           menu.display() #Displays menu
       pygame.display.update()

if __name__=='__main__':
   run()



import random
import os
import pygame
from pygame.locals import *
import time
from Patoman import Matriz


main_dir = os.path.split(os.path.abspath(__file__))[0]
def load_img(name):
   path=os.path.join(main_dir,"imgs",name)
   return pygame.image.load(path)

matriz = Matriz.mat();

UP = 0
LEFT = 1
DOWN = 2
RIGHT = 3
NONE = 4

Inky = 0
Blinky = 1
Clyde = 2
Pinky = 3

SCATTER = 0
CHASE = 1
EATEN = 2
FRIGHTENED = 3
LEAVE = 4
WAITING = 5

TS = 1
limit = len(matriz[0])

imgs =  [
[
[[load_img('InkyUp1.png'  ), load_img('InkyUp2.png'  )], [load_img('InkyLeft1.png'  ), load_img('InkyLeft2.png'  )], [load_img('InkyDown1.png'  ), load_img('InkyDown2.png'  )], [load_img('InkyRight1.png'  ), load_img('InkyRight2.png'  )]],
[[load_img('BlinkyUp1.png'), load_img('BlinkyUp2.png')], [load_img('BlinkyLeft1.png'), load_img('BlinkyLeft2.png')], [load_img('BlinkyDown1.png'), load_img('BlinkyDown2.png')], [load_img('BlinkyRight1.png'), load_img('BlinkyRight2.png')]],
[[load_img('ClydeUp1.png' ), load_img('ClydeUp2.png' )], [load_img('ClydeLeft1.png' ), load_img('ClydeLeft2.png' )], [load_img('ClydeDown1.png' ), load_img('ClydeDown2.png' )], [load_img('ClydeRight1.png' ), load_img('ClydeRight2.png' )]],
[[load_img('PinkyUp1.png' ), load_img('PinkyUp2.png' )], [load_img('PinkyLeft1.png' ), load_img('PinkyLeft2.png' )], [load_img('PinkyDown1.png' ), load_img('PinkyDown2.png' )], [load_img('PinkyRight1.png' ), load_img('PinkyRight2.png' )]],
],
[
[[load_img('InkyUp1.png'  ), load_img('InkyUp2.png'  )], [load_img('InkyLeft1.png'  ), load_img('InkyLeft2.png'  )], [load_img('InkyDown1.png'  ), load_img('InkyDown2.png'  )], [load_img('InkyRight1.png'  ), load_img('InkyRight2.png'  )]],
[[load_img('BlinkyUp1.png'), load_img('BlinkyUp2.png')], [load_img('BlinkyLeft1.png'), load_img('BlinkyLeft2.png')], [load_img('BlinkyDown1.png'), load_img('BlinkyDown2.png')], [load_img('BlinkyRight1.png'), load_img('BlinkyRight2.png')]],
[[load_img('ClydeUp1.png' ), load_img('ClydeUp2.png' )], [load_img('ClydeLeft1.png' ), load_img('ClydeLeft2.png' )], [load_img('ClydeDown1.png' ), load_img('ClydeDown2.png' )], [load_img('ClydeRight1.png' ), load_img('ClydeRight2.png' )]],
[[load_img('PinkyUp1.png' ), load_img('PinkyUp2.png' )], [load_img('PinkyLeft1.png' ), load_img('PinkyLeft2.png' )], [load_img('PinkyDown1.png' ), load_img('PinkyDown2.png' )], [load_img('PinkyRight1.png' ), load_img('PinkyRight2.png' )]],
],
[
[[load_img('EatenUp.png'), load_img('EatenUp.png')], [load_img('EatenLeft.png'), load_img('EatenLeft.png')], [load_img('EatenDown.png'), load_img('EatenDown.png')], [load_img('EatenRight.png'), load_img('EatenRight.png')]],
[[load_img('EatenUp.png'), load_img('EatenUp.png')], [load_img('EatenLeft.png'), load_img('EatenLeft.png')], [load_img('EatenDown.png'), load_img('EatenDown.png')], [load_img('EatenRight.png'), load_img('EatenRight.png')]],
[[load_img('EatenUp.png'), load_img('EatenUp.png')], [load_img('EatenLeft.png'), load_img('EatenLeft.png')], [load_img('EatenDown.png'), load_img('EatenDown.png')], [load_img('EatenRight.png'), load_img('EatenRight.png')]],
[[load_img('EatenUp.png'), load_img('EatenUp.png')], [load_img('EatenLeft.png'), load_img('EatenLeft.png')], [load_img('EatenDown.png'), load_img('EatenDown.png')], [load_img('EatenRight.png'), load_img('EatenRight.png')]],
],
[
[[load_img('Frightened1.png'), load_img('Frightened2.png')], [load_img('Frightened1.png'), load_img('Frightened2.png')], [load_img('Frightened1.png'), load_img('Frightened2.png')], [load_img('Frightened1.png'), load_img('Frightened2.png')]],
[[load_img('Frightened1.png'), load_img('Frightened2.png')], [load_img('Frightened1.png'), load_img('Frightened2.png')], [load_img('Frightened1.png'), load_img('Frightened2.png')], [load_img('Frightened1.png'), load_img('Frightened2.png')]],
[[load_img('Frightened1.png'), load_img('Frightened2.png')], [load_img('Frightened1.png'), load_img('Frightened2.png')], [load_img('Frightened1.png'), load_img('Frightened2.png')], [load_img('Frightened1.png'), load_img('Frightened2.png')]],
[[load_img('Frightened1.png'), load_img('Frightened2.png')], [load_img('Frightened1.png'), load_img('Frightened2.png')], [load_img('Frightened1.png'), load_img('Frightened2.png')], [load_img('Frightened1.png'), load_img('Frightened2.png')]],
],
[
[[load_img('InkyUp1.png'  ), load_img('InkyUp2.png'  )], [load_img('InkyLeft1.png'  ), load_img('InkyLeft2.png'  )], [load_img('InkyDown1.png'  ), load_img('InkyDown2.png'  )], [load_img('InkyRight1.png'  ), load_img('InkyRight2.png'  )]],
[[load_img('BlinkyUp1.png'), load_img('BlinkyUp2.png')], [load_img('BlinkyLeft1.png'), load_img('BlinkyLeft2.png')], [load_img('BlinkyDown1.png'), load_img('BlinkyDown2.png')], [load_img('BlinkyRight1.png'), load_img('BlinkyRight2.png')]],
[[load_img('ClydeUp1.png' ), load_img('ClydeUp2.png' )], [load_img('ClydeLeft1.png' ), load_img('ClydeLeft2.png' )], [load_img('ClydeDown1.png' ), load_img('ClydeDown2.png' )], [load_img('ClydeRight1.png' ), load_img('ClydeRight2.png' )]],
[[load_img('PinkyUp1.png' ), load_img('PinkyUp2.png' )], [load_img('PinkyLeft1.png' ), load_img('PinkyLeft2.png' )], [load_img('PinkyDown1.png' ), load_img('PinkyDown2.png' )], [load_img('PinkyRight1.png' ), load_img('PinkyRight2.png' )]],
]
		]
		
frightened_img = [
[load_img('Frightened1.png'), load_img('Frightened2.png')],
[load_img('FrightenedBlink1.png'), load_img('FrightenedBlink2.png')]
				 ]

Blinky_img = pygame.Surface((16,16))
Blinky_img.fill((255,0,0))

Inky_img = pygame.Surface((16,16))
Inky_img.fill((19,249,226))

Clyde_img = pygame.Surface((16,16))
Clyde_img.fill((209,239,13))

Pinky_img = pygame.Surface((16,16))
Pinky_img.fill((218,15,245))

Ajuste = 6

class Ghost(pygame.sprite.Sprite):
	
   def __init__(self, type_ghost):
      self.ghost = type_ghost

      if(self.ghost == Inky):
         self.pos = (184, 280)
      if(self.ghost == Blinky):
         self.pos = (216, 224)
      if(self.ghost == Clyde):
         self.pos = (248, 280)
      if(self.ghost == Pinky):
         self.pos = (216 * TS, 280)

      self.mode = LEAVE
 
      pygame.sprite.Sprite.__init__(self)
      self.direct = DOWN
      self.change = 0
      self.img = imgs[self.mode][self.ghost][self.direct][self.change]
      self.rect = self.img.get_rect()
      self.rect.height = 16
      self.rect.width = 16
      self.rect.x = self.pos[0]
      self.rect.y = self.pos[1]
      self.time = 0
      self.timer = time.time()
      self.normal_speed=0
      self.slow_speed=0
      self.speed=0

      if(self.ghost == Pinky):
         self.mode = LEAVE
      if(self.ghost == Blinky):
         self.mode = SCATTER
      if(self.ghost == Clyde):
         self.mode = WAITING
         self.waiting_moviments = -30
         self.up = True
      if(self.ghost == Inky):
         self.mode = WAITING
         self.waiting_moviments = 0
         self.up = True

      self.frightened = False

      self.box = False

      self.moviments = 0
	
   def dis(self, x, y):
      return (self.target[0] - x) ** 2 + (self.target[1] - y) ** 2
  
   def change_direct(self):
      if(self.direct == UP):
         self.direct = DOWN
      elif(self.direct == DOWN):
         self.direct = UP
      elif(self.direct == RIGHT):
         self.direct = LEFT
      elif(self.direct == LEFT):
         self.direct = RIGHT
  
   def set_image(self):
      real_mode = self.mode
          
      if(self.mode == WAITING):
         if(self.frightened):
            self.mode = FRIGHTENED
         else:
            self.mode = SCATTER
      if(self.mode == LEAVE):
         if(self.frightened):
            self.mode = FRIGHTENED
       
      if(self.mode == FRIGHTENED and time.time() - self.timer >= 10):
         self.img = frightened_img[self.change][self.change]
      else:
         self.img = imgs[self.mode][self.ghost][self.direct][self.change]
          
      self.rect.x = self.pos[0]
      self.rect.y = self.pos[1]
          
      self.mode = real_mode
   
   def reset_time(self):
      self.timer = time.time()
   
   def move(self, x, y, direct, redx, redy, coins_left):
      possible_moviments = []
        
      self.change = (self.change + 1) % 2
          
      if(self.mode == SCATTER):
         self.speed=self.normal_speed
         if(self.ghost == Inky):
            self.target = (568, 440)
         if(self.ghost == Blinky):
            self.target = (-8, 408)
         if(self.ghost == Clyde):
            self.target = (568, 0)
         if(self.ghost == Pinky):
            self.target = (-8, 32)
            
         #Find possible Moviments for Ink
         if(self.direct != LEFT and matriz[self.pos[1]][(self.pos[0] + 1) % limit] != 1):
            possible_moviments.append([self.dis(self.pos[1], (self.pos[0] + 1) % limit), RIGHT, self.pos[1], (self.pos[0] + 1) % limit])
         if(self.direct != RIGHT and matriz[self.pos[1]][(self.pos[0] - 1 + limit) % limit] != 1):
            possible_moviments.append([self.dis(self.pos[1], (self.pos[0] - 1 + limit) % limit), LEFT, self.pos[1], (self.pos[0] - 1 + limit) % limit])
         if(self.direct != UP and matriz[self.pos[1] + 1][self.pos[0]] != 1 and matriz[self.pos[1] + 1][self.pos[0]] != 4):
            possible_moviments.append([self.dis(self.pos[1] + 1, self.pos[0]), DOWN, self.pos[1] + 1, self.pos[0]])
         if(self.direct != DOWN and matriz[self.pos[1] - 1][self.pos[0]] != 1):
            can = True
                        
            if((self.pos[1] // TS - 1 == 223 or self.pos[1] // TS - 1 == 415) and (self.pos[0] // TS == 192 or self.pos[0] // TS == 240)):
               can = False
            if(can):
               possible_moviments.append([self.dis(self.pos[1] - 1, self.pos[0]), UP, self.pos[1] - 1, self.pos[0]])
            
         #Calculate better moviment
         possible_moviments.sort()
                  
         self.pos = (possible_moviments[0][3] * TS, possible_moviments[0][2] * TS)
         self.direct = possible_moviments[0][1]
                  
         if(time.time() - self.timer >= 8):
            self.mode = CHASE
            self.change_direct()
            self.reset_time()
      elif(self.mode == FRIGHTENED):
         self.speed=self.slow_speed
            
         #Find possible Moviments
         if(self.direct != LEFT and matriz[self.pos[1] // TS][(self.pos[0] // TS + 1) % limit] != 1):
            possible_moviments.append([self.pos[1] // TS, (self.pos[0] // TS + 1) % limit, RIGHT])
         if(self.direct != RIGHT and matriz[self.pos[1] // TS][self.pos[0] // TS - 1] != 1):
            possible_moviments.append([self.pos[1] // TS, (self.pos[0] // TS - 1 + limit) % limit, LEFT])
         if(self.direct != UP and matriz[self.pos[1] // TS + 1][self.pos[0] // TS] != 1 and matriz[self.pos[1] // TS + 1][self.pos[0] // TS] != 4):
            possible_moviments.append([self.pos[1] // TS + 1, self.pos[0] // TS, DOWN])
         if(self.direct != DOWN and matriz[self.pos[1] // TS - 1][self.pos[0] // TS] != 1):
            can = True
                          
            if((self.pos[1] // TS - 1 == 223 or self.pos[1] // TS - 1 == 415) and (self.pos[0] // TS == 192 or self.pos[0] // TS == 240)):
               can = False
            if(can):
               possible_moviments.append([self.pos[1] // TS - 1, self.pos[0] // TS, UP])
                  
         #Choose a random direction
         random_direct = random.choice(possible_moviments)
         self.pos = (random_direct[1] * TS, random_direct[0] * TS)
         self.direct = random_direct[2]
            
         if(time.time() - self.timer >= 12):
            if(coins_left < 80 and self.ghost == Blinky):
               self.mode = CHASE
            else:
               self.mode = SCATTER
                
               self.change_direct()
               self.reset_time()
      elif(self.mode == EATEN):
         self.speed=250
         if(self.box):
            if(self.ghost == Inky):
               self.target = (280, 184)
            if(self.ghost == Blinky):
               self.target = (280, 216)
            if(self.ghost == Clyde):
               self.target = (280, 248)
            if(self.ghost == Pinky):
               self.target = (280, 216)
         else: 
            self.target = (224, 216)
                  
         #Find possible Moviments for Ink
         if(self.direct != LEFT and matriz[self.pos[1]][(self.pos[0] + 1) % limit] != 1):
            possible_moviments.append([self.dis(self.pos[1], (self.pos[0] + 1) % limit), RIGHT, self.pos[1], (self.pos[0] + 1) % limit])
         if(self.direct != RIGHT and matriz[self.pos[1]][(self.pos[0] - 1 + limit) % limit] != 1):
            possible_moviments.append([self.dis(self.pos[1], (self.pos[0] - 1 + limit) % limit), LEFT, self.pos[1], (self.pos[0] - 1 + limit) % limit])
         if(self.direct != UP and matriz[self.pos[1] + 1][self.pos[0]] != 1):
            possible_moviments.append([self.dis(self.pos[1] + 1, self.pos[0]), DOWN, self.pos[1] + 1, self.pos[0]])
         if(self.direct != DOWN and matriz[self.pos[1] - 1][self.pos[0]] != 1):
            can = True
                        
            if((self.pos[1] // TS - 1 == 223 or self.pos[1] // TS - 1 == 415) and (self.pos[0] // TS == 192 or self.pos[0] // TS == 240)):
               can = False
            if(can):
               possible_moviments.append([self.dis(self.pos[1] - 1, self.pos[0]), UP, self.pos[1] - 1, self.pos[0]])
                  
         #Calculate better moviment
         possible_moviments.sort()
                  
         self.pos = (possible_moviments[0][3] * TS, possible_moviments[0][2] * TS)
         self.direct = possible_moviments[0][1]
            
         if(self.target[0] == (self.pos[1] // TS) and self.target[1] == (self.pos[0] // TS)):
                        
            if(self.box):
               self.mode = LEAVE
               self.change_direct()
            else:
               self.box = True
         
      elif(self.mode == CHASE):
         self.speed=self.normal_speed
         
         if(self.ghost == Inky):
            self.target = (x - (redx - x), y - (redy - y))
         if(self.ghost == Blinky):
            self.target = (x, y)
         if(self.ghost == Clyde):
            self.target = (x, y)
            
            if(self.dis(self.pos[1] // TS, self.pos[0] // TS) < 256):
               self.target = (568, 0)
         if(self.ghost == Pinky):
            if(direct == LEFT):
               self.target = (x, y - 64)
            if(direct == RIGHT):
               self.target = (x, y + 64)
            if(direct == DOWN):
               self.target = (x + 64, y)
            if(direct == UP):
               self.target = (x - 64, y - 64)
         #Find possible Moviments for Ink
         if(self.direct != LEFT and matriz[self.pos[1]][(self.pos[0] + 1) % limit] != 1):
            possible_moviments.append([self.dis(self.pos[1], (self.pos[0] + 1) % limit), RIGHT, self.pos[1], (self.pos[0] + 1) % limit])
         if(self.direct != RIGHT and matriz[self.pos[1]][(self.pos[0] - 1 + limit) % limit] != 1):
            possible_moviments.append([self.dis(self.pos[1], (self.pos[0] - 1 + limit) % limit), LEFT, self.pos[1], (self.pos[0] - 1 + limit) % limit])
         if(self.direct != UP and matriz[self.pos[1] + 1][self.pos[0]] != 1 and matriz[self.pos[1] + 1][self.pos[0]] != 4):
            possible_moviments.append([self.dis(self.pos[1] + 1, self.pos[0]), DOWN, self.pos[1] + 1, self.pos[0]])
         if(self.direct != DOWN and matriz[self.pos[1] - 1][self.pos[0]] != 1):
            can = True
                        
            if((self.pos[1] // TS - 1 == 223 or self.pos[1] // TS - 1 == 415) and (self.pos[0] // TS == 192 or self.pos[0] // TS == 240)):
               can = False
            if(can):
               possible_moviments.append([self.dis(self.pos[1] - 1, self.pos[0]), UP, self.pos[1] - 1, self.pos[0]])
              
         #Calculate better moviment
         possible_moviments.sort()
              
         self.pos = (possible_moviments[0][3] * TS, possible_moviments[0][2] * TS)
         self.direct = possible_moviments[0][1]
              
         if(time.time() - self.timer >= 20):
            if(coins_left < 80 and self.ghost == Blinky):
               self.mode = CHASE
            else:
               self.mode = SCATTER
               self.change_direct()
            self.reset_time()
      elif(self.mode == LEAVE):
         self.speed=self.normal_speed
         self.target = (224, 216)
                  
         #Find possible Moviments
         if(matriz[self.pos[1] // TS][(self.pos[0] // TS + 1) % limit] != 1):
            possible_moviments.append([self.dis(self.pos[1] // TS, (self.pos[0] // TS + 1) % limit), RIGHT, self.pos[1] // TS, (self.pos[0] // TS + 1) % limit])
         if(matriz[self.pos[1] // TS][self.pos[0] // TS - 1] != 1):
            possible_moviments.append([self.dis(self.pos[1] // TS, (self.pos[0]//TS - 1 + limit) % limit), LEFT, self.pos[1] // TS, (self.pos[0] // TS - 1 + limit) % limit])
         if(matriz[self.pos[1] // TS + 1][self.pos[0] // TS] != 1):
            possible_moviments.append([self.dis(self.pos[1] // TS + 1, self.pos[0] // TS), DOWN, self.pos[1] // TS + 1, self.pos[0] // TS])
         if(matriz[self.pos[1] // TS - 1][self.pos[0] // TS] != 1):
                     
            can = True
                          
            if((self.pos[1] // TS - 1 == 223 or self.pos[1] // TS - 1 == 415) and (self.pos[0] // TS == 192 or self.pos[0] // TS == 240)):
               can = False
            if(can):
               possible_moviments.append([self.dis(self.pos[1] // TS - 1, self.pos[0] // TS), UP, self.pos[1] // TS - 1, self.pos[0] // TS])
                  
         #Calculate better moviment
         possible_moviments.sort()
                  
         self.pos = (possible_moviments[0][3] * TS, possible_moviments[0][2] * TS)
         self.direct = possible_moviments[0][1]
              
         if(self.frightened and time.time() - self.timer >= 20):
            self.frightened = False
              
         #Check if the ghost left the box
         if(self.target[0] == (self.pos[1] // TS) and self.target[1] == (self.pos[0] // TS)):
            if(self.frightened):
               self.mode = FRIGHTENED
               self.frightened = False
            elif(coins_left < 80 and self.ghost == Blinky):
               self.mode = CHASE
               self.reset_time()
            else:
               self.mode = SCATTER
               self.reset_time()
        
      elif(self.mode == WAITING):
         self.speed=self.normal_speed
         
         if(self.up):
            if(self.pos[1] // TS == 264):
               self.up = False
               self.pos = (self.pos[0], self.pos[1] + TS)
               self.direct = DOWN
            else:
               self.pos = (self.pos[0], self.pos[1] - TS)
         elif(not self.up):
            if(self.pos[1] // TS == 280):
               self.up = True
               self.waiting_moviments += 1
                                  
               if(self.waiting_moviments >= 10):
                  self.mode = LEAVE
               else:
                  self.pos = (self.pos[0], self.pos[1] - TS)
                  self.direct = UP
            else:
               self.pos = (self.pos[0], self.pos[1] + TS)
                
               if(time.time() - self.timer >= 12):
                  self.frightened = False
        
      self.set_image()



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="Pato-man",
    version="0.9.3",
    author="Abraão Caiana de Freitas",
    author_email="abraao.freitas@ccc.ufcg.edu.br",
    description="Pac-man with ducks",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/AbraaoCF/Pato-man",
    packages=setuptools.find_packages(),
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'pygame'
        ],
    python_requires='>=3.6',
    entry_points={
        'console_scripts': [
            'patoman=Patoman.menu:run'
            ]
        }
)



import pygame
import os
from pygame.locals import *
import shelve
import time
from Patoman import Matriz
from Patoman import menu
from Patoman import Ghost


main_dir = os.path.split(os.path.abspath(__file__))[0]
shelve_path=os.path.join(main_dir,"score.txt")

#Directions
UP    = 0
LEFT  = 1
DOWN  = 2
RIGHT = 3

#Tile Size (Define the number of pixels on each tile)
TS = 1

#Defining Ghosts
Inky = 0
Blinky = 1
Clyde = 2
Pinky = 3

#Defining Ghosts' modes
SCATTER = 0
CHASE = 1
EATEN = 2
FRIGHTENED = 3
LEAVE = 4
WAITING = 5

#Adjust sprite position
Ajuste = 6
   
def load_img(name):
   path=os.path.join(main_dir,"imgs",name)
   return pygame.image.load(path)

def load_sound(name,volume):
   path=os.path.join(main_dir,"sounds",name)
   this_sound = pygame.mixer.Sound(path)
   this_sound.set_volume(volume)
   return this_sound

def play_music(name,volume):
   path=os.path.join(main_dir,"sounds",name)
   pygame.mixer.music.load(path)
   pygame.mixer.music.set_volume(volume)
   pygame.mixer.music.play(loops=-1)

def load_font(name,size):
   path=os.path.join(main_dir,"fonts",name)
   return pygame.font.Font(path,size)

def go_to_menu():
   running=False
   pygame.mixer.music.unload()
   pygame.mixer.pause()
   menu.run()

def setup(volume,is_gamer):
   #Initializing modules
   pygame.init()
   if not pygame.mixer.get_init():
     pygame.mixer.init()
        
   #Loads images
   background = load_img('background.png')
   background_white = load_img('background_white.png')
   background_open = load_img('background_open.png')
   power_img = load_img('corn.png')
   coin_img = load_img('bit.png')
   patoFC = load_img('patoFC.png')
   patoFD = load_img('patoFD.png')
   patoFB = load_img('patoFB.png')
   patoFE = load_img('patoFE.png')
   patoAC = load_img('patoAC.png')
   patoAD = load_img('patoAD.png')
   patoAB = load_img('patoAB.png')
   patoAE = load_img('patoAE.png')
   arrow = load_img('arrow.png')

   #Loads the sounds
   eat1 = load_sound('pac_chomp_one.wav',volume)
   eat2 = load_sound('pac_chomp_two.wav',volume)
   death = load_sound('death.wav',volume)
   eat_ghost = load_sound('eat.wav',volume)
   oneup = load_sound('1up.wav',volume)

   #loads the fonts
   font=load_font("emulogic.ttf",16) 
   big_font=load_font("emulogic.ttf",32)

   #Screen
   screen = pygame.display.set_mode((8 * 56, 8 * 72))
   pygame.display.set_caption("Pato-man")
   #Clock Speed
   clock = pygame.time.Clock()

   def game(current_lives,current_score,player_normal_speed,player_fast_speed,ghost_normal_speed,ghost_slow_speed,is_gamer):
      def reset():
         #Inital screen appearence
         screen.fill((0, 0, 0))
         score.display()
         lives.display()
         screen.blit(background,(0,6*8))
         consumables.draw(screen) #displays the consumables
         screen.blit(font.render("ready!",False,pygame.Color("yellow")),(8*22,8*40))

         for i in range(4):
            if i==Inky:
               phantom[i].pos=(23*8,35*8)
               phantom[i].mode = WAITING
               phantom[i].waiting_movements = -30
               phantom[i].up = True
            elif i==Blinky:
               phantom[i].pos=(27*8,28*8)
               phantom[i].mode=SCATTER
            elif i==Clyde:
               phantom[i].pos=(31*8,35*8)
               phantom[i].mode = WAITING
               phantom[i].waiting_movements = 0
               phantom[i].up = True               
            elif i==Pinky:
               phantom[i].pos=(27*8,35*8)
               phantom[i].mode=LEAVE
            phantom[i].frightened = False
            phantom[i].box = False
            phantom[i].moviments = 0
            phantom[i].direct = DOWN
            phantom[i].change = 0
            phantom[i].rect = phantom[i].img.get_rect()
            phantom[i].rect.height = 16
            phantom[i].rect.width = 16
            phantom[i].rect.x = phantom[i].pos[0]-Ajuste
            phantom[i].rect.y = phantom[i].pos[1]-Ajuste
            phantom[i].set_image()
            screen.blit(phantom[i].img, (phantom[i].pos[0]-Ajuste, phantom[i].pos[1]-Ajuste))
         player.pos=(27*8,52*8)
         player.direct=LEFT
         player.display()
         pygame.display.update()
         pygame.time.wait(2150)
         pygame.mixer.music.unpause()

      def pause_game():
         running=False
         pygame.mixer.music.pause()
         pygame.mixer.pause()
         
         pause_screen=pygame.Surface((8*56,8*72))
         pause_screen.set_alpha(100)
         cursor_index=0
         positions=[(8*9,8*28+4),(8*1,8*34+4)]
         while not running:

            clock.tick(60)
            for event in pygame.event.get():
               #Quit game
               if event.type == pygame.QUIT:
                  pygame.quit()

               #Directions keys
               if event.type == KEYDOWN:
                  if event.key == K_ESCAPE:
                     running=True
                     pygame.mixer.music.unpause()
                     pygame.mixer.unpause()

                  if event.key == K_RETURN:

                     if cursor_index==0:
                        running=True
                        pygame.mixer.music.unpause()
                        pygame.mixer.unpause()
                     else:
                        go_to_menu()
                         
                  if event.key == K_UP:
                     cursor_index+=1
                     if cursor_index==2:
                        cursor_index=0
                      
                  if event.key == K_DOWN:
                     cursor_index-=1
                     if cursor_index==-1:
                        cursor_index=1

            screen.fill((0, 0, 0))
            score.display()
            lives.display()
            screen.blit(background,(0, 6*8))
            consumables.draw(screen)
            player.display()  

            screen.blit(pause_screen,(0,0))
            screen.blit(big_font.render("PAUSED",False,pygame.Color("White")),(8*16,8*16))
            screen.blit(big_font.render("continue", False, pygame.Color('White')),(8*12,8*28))
            screen.blit(big_font.render("back to menu", False, pygame.Color('White')),(8*4,8*34))
            screen.blit(arrow,positions[cursor_index])

            pygame.display.update()
      
      def win_game():
         running=False
         pygame.mixer.music.pause()
         pygame.mixer.pause()
         
         path=os.path.join(main_dir,"sounds",'win_sound.wav')
         pygame.mixer.music.load(path)
         pygame.mixer.music.set_volume(volume)
         pygame.mixer.music.play(loops=1)
         
         pause_screen=pygame.Surface((8*56,8*72))
         pause_screen.set_alpha(100)
         cursor_index=0
         positions=[(8*1,8*34+4)]
         while not running:
            
            clock.tick(60)
            for event in pygame.event.get():
               #Quit game
               if event.type == pygame.QUIT:
                  pygame.quit()

               #Directions keys
               if event.type == KEYDOWN:
                  if event.key == K_RETURN:
                     go_to_menu()
                     
            screen.fill((0, 0, 0))
            score.display()
            lives.display()
            screen.blit(background,(0, 6*8))
            consumables.draw(screen)
            player.display() 
            
            screen.blit(pause_screen,(0,0))
            screen.blit(big_font.render("YOU WIN", False, pygame.Color('Green')),(8*12,8*28))
            screen.blit(big_font.render("back to menu", False, pygame.Color('White')),(8*4,8*34))
            screen.blit(arrow,positions[cursor_index])

            pygame.display.update()
      
      class Score:

         def __init__(self,current_score,gamer):
            self.score=current_score #starting score
            self.is_gamer=gamer
            self.limit=int(self.score)//10000 + 1
            
            d = shelve.open(shelve_path)#open high score memory
            self.high_score='00'
            if not self.is_gamer and 'score' in d:
               self.high_score=d['score'] #Starts as last high value if played before (modo normal)
            elif self.is_gamer and 'score_gamer' in d:
               self.high_score=d['score_gamer'] #Starts as last high value if played before (modo gamer)
            d.close() #close high score memory

            self.high_score_text=font.render(self.high_score,False,pygame.Color("white")) #makes starter high score text 
            self.score_text=font.render(self.score,False,pygame.Color("white")) #makes starter score text

         def display(self):
            screen.blit(font.render("high score",False,pygame.Color("white")),(144,0)) #display "high score" on screen
            screen.blit(self.score_text,(112-16*len(self.score),16)) #display current score on screen
            screen.blit(self.high_score_text,(272-16*len(self.high_score),16)) #display current high score on screen


         def add(self,num):
            new_score=int(self.score)+num
            if new_score>=10000*self.limit:
               lives.add()
               self.limit+=1
               oneup.play() #som de 1up
            self.score=str(new_score) #updates current score
            if int(self.score)>int(self.high_score): #checks if we need new high score
               d = shelve.open(shelve_path) #open high score memory
               if not self.is_gamer:
                  d['score'] = self.score #update high score memory (modo normal)
               if self.is_gamer:
                  d['score_gamer'] = self.score #update high score memory (modo gamer)

               d.close() #close high score memory
               self.high_score=self.score #gets new high score value
               self.high_score_text=font.render(self.high_score,False,pygame.Color("white")) #makes new high score text
            
            self.score_text=font.render(self.score,False,pygame.Color("white")) #makes new score text

      class Lives:

         def __init__(self,current_lives,gamer):
            self.lives=current_lives
            self.is_gamer=gamer

         def add(self):
            if self.lives<5 and not is_gamer:
               self.lives+=1

         def remove(self):
            self.lives-=1
            if self.lives==0:
               #Game over screen
               screen.fill((0, 0, 0))
               score.display()
               lives.display()
               screen.blit(background,(0,6*8))
               consumables.draw(screen)
               screen.blit(font.render("Game over",False,pygame.Color("red")),(8*19,8*40))
               pygame.display.update()
               pygame.time.wait(3500)
               go_to_menu()

         def display(self):
            for i in range(self.lives):
               screen.blit(patoAD,(8+i*32,68*8+4))
               
            
      class Coins(pygame.sprite.Sprite):

         def __init__(self):
            pygame.sprite.Sprite.__init__(self)
            self.image = coin_img
            self.rect = self.image.get_rect()

      class Power(pygame.sprite.Sprite):

         def __init__(self):
            pygame.sprite.Sprite.__init__(self)
            self.image = power_img
            self.rect = self.image.get_rect()

      class Player(pygame.sprite.Sprite):

         def __init__(self):
            self.pos = (27*8,52*8)

            self.direct = LEFT
            self.last_direct = LEFT
            self.memory_direct = LEFT
            self.change_direct = 0

            self.img_index=0
            self.imgs = [patoFC,patoFD,patoFB,patoFE,patoAC,patoAD,patoAB,patoAE]
            self.image = patoFC
            self.aberto=False
            
            
            self.rect = self.image.get_rect()

            self.rect.x = self.pos[0]
            self.rect.y = self.pos[1]
            self.rect.height = 16
            self.rect.width = 16

            self.eaten_phantom = 0
            self.set_image()
            self.count = True

            self.time=0
            self.normal_speed=player_normal_speed
            self.fast_speed=player_fast_speed
            self.speed=self.normal_speed
         def set_xy(self):
            self.rect.x = self.pos[0]
            self.rect.y = self.pos[1]

         def change_sound(self):
            #change chomp sound
            if self.count:
               eat1.play()
            else:
               eat2.play()
            self.count = not self.count
            
         def move(self):
            
            #Try (only one time) to do the memory direction
            if self.change_direct == 1:
               self.direct = self.memory_direct

            if self.direct == UP:
               self.pos = (self.pos[0],self.pos[1]-TS)
            if self.direct == RIGHT:
               self.pos = (self.pos[0]+TS,self.pos[1])
            if self.direct == DOWN:
               self.pos = (self.pos[0],self.pos[1]+TS)
            if self.direct == LEFT:
               self.pos=(self.pos[0]-TS,self.pos[1])
                   
            if self.pos[1] == 34*8:
               if self.pos[0] < 0:
                  self.move_absolute(screen.get_width()-TS,self.pos[1])
               elif self.pos[0] >= screen.get_width():
                  self.move_absolute(0,self.pos[1])

            if matriz[self.pos[1]][self.pos[0]] == 1:
               if self.direct == UP:
                  self.pos = (self.pos[0], self.pos[1] + TS)
               if self.direct == RIGHT:
                  self.pos = (self.pos[0] - TS, self.pos[1])
               if self.direct == DOWN:
                  self.pos = (self.pos[0], self.pos[1] - TS)
               if self.direct == LEFT:
                  self.pos = (self.pos[0] + TS, self.pos[1])

               if self.last_direct != self.direct:

                  #If direction is invalid, memory_direction save it
                  self.memory_direct = self.direct
                  self.change_direct += 1

                  self.direct = self.last_direct
                  self.move()

            self.last_direct = self.direct
            self.set_xy()
            self.set_image()

         def move_absolute(self,x,y):
            self.pos=(x,y)
            self.set_xy()

         def death(self):
            rotate = pygame.transform.rotate
            for frame in range(-1,7):
               screen.fill((0, 0, 0))
               score.display()
               lives.display()
               screen.blit(background,(0,6*8))
               consumables.draw(screen)
               screen.blit(rotate(patoAD,-(frame*90+90)),(self.pos[0]-4,self.pos[1]-4))
               pygame.display.update()
               pygame.time.wait(150)
            pygame.time.wait(800)
            
         def set_image(self):
            self.img_index=0
            if self.aberto:
               self.img_index+=4
            if self.direct==RIGHT:
               self.img_index+=1
            elif self.direct==DOWN:
               self.img_index+=2
            elif self.direct==LEFT:
               self.img_index+=3
            self.set_xy()

         def display(self):
            screen.blit(self.imgs[self.img_index],(self.pos[0]-4,self.pos[1]-4))
            
         def collision(self):
            
            phantom_player_collision = pygame.sprite.spritecollide(player, phantom_list, False)
            
            for ghost in phantom_player_collision:
                
               if (ghost.mode == EATEN):
                  continue
               if (ghost.mode == FRIGHTENED):
                    
                  phantom[ghost.ghost].mode = EATEN
                  eat_ghost.play()

                  #display everything but pacman and eaten ghost
                  screen.fill((0, 0, 0))
                  score.display()
                  lives.display()
                  screen.blit(background,(0, 6*8))                    
                  consumables.draw(screen)
                  for i in range(4):
                     if i!=ghost.ghost:
                        screen.blit(phantom[i].img, (phantom[i].pos[0]-Ajuste, phantom[i].pos[1]-Ajuste))

                  #display score got
                  screen.blit(load_img('score'+str(self.eaten_phantom)+'.png'),(ghost.pos[0]-7,ghost.pos[1]))
                  pygame.display.update()

                  pygame.time.wait(1000)

                  #displays everything normally
                  screen.fill((0, 0, 0))
                  score.display()
                  lives.display()
                  screen.blit(background,(0, 6*8))                    
                  consumables.draw(screen)
                  for i in range(4):
                     screen.blit(phantom[i].img, (phantom[i].pos[0]-Ajuste, phantom[i].pos[1]-Ajuste))
                  self.display()
                  pygame.display.update()
                    
                  phantom[ghost.ghost].box = False
                  score.add(200 * (2 ** self.eaten_phantom))
                  self.eaten_phantom += 1
               else:
                  pygame.mixer.music.pause()
                  pygame.time.wait(750)
                  death.play()
                  self.death() #death animation
                  lives.remove()
                  reset()

      #Stage 1 matrix. Holds information on active game state
      #State 0 --> Empty
      #State 1 --> Wall
      #State 2 --> Coin
      #State 3 --> Power
      matriz = Matriz.mat()

      #Create the groups of sprites
      consumables = pygame.sprite.Group() 
      phantom = [Ghost.Ghost(0), Ghost.Ghost(1), Ghost.Ghost(2), Ghost.Ghost(3)]
      phantom_list = pygame.sprite.Group()
      phantom_list.add(phantom[Inky  ])
      phantom_list.add(phantom[Blinky])
      phantom_list.add(phantom[Clyde ])
      phantom_list.add(phantom[Pinky ])

      for i in range(4):
         phantom[i].normal_speed=ghost_normal_speed
         phantom[i].slow_speed=ghost_slow_speed
         phantom[i].speed=ghost_normal_speed
      
      player = Player()
      score = Score(current_score,is_gamer)
      lives = Lives(current_lives,is_gamer)

      #Create the sprites of coins and powers and put them in the respective groups
      for i in range(len(matriz)):
         for j in range(len(matriz[0])):
            coins = Coins()
            power = Power()

            if (matriz[i][j] == 2):
               coins.rect.x = j
               coins.rect.y = i
               consumables.add(coins)

            if (matriz[i][j] == 3):
               power.rect.x = j
               power.rect.y = i
               consumables.add(power)

      #Inital screen appearence
      screen.fill((0, 0, 0))
      score.display()
      lives.display()
      screen.blit(background,(0,6*8))
      consumables.draw(screen) #displays the consumables
      screen.blit(font.render("ready!",False,pygame.Color("yellow")),(8*22,8*40))
      screen.blit(font.render("player one",False,pygame.Color("cyan")),(8*18,8*28))
      pygame.display.update()

      #Plays the beginning sound
      play_music('begin.wav',volume)
      pygame.time.wait(2150)

      #initial screen appearence 2
      screen.blit(background,(0,6*8))
      consumables.draw(screen) #displays the consumables
      for i in range(4):#displays ghosts       
         screen.blit(phantom[i].img, (phantom[i].pos[0] - Ajuste, phantom[i].pos[1] - Ajuste))
      player.display()
      pygame.display.update()

      #Waits until sound is over so game can start
      pygame.time.wait(2150)
      play_music('music1.wav',volume)
      level_music = 'music1.wav'
      now_music = 'music1.wav'

      running = True
      player_time=0

      while running:
            
         clock.tick()
           
         for event in pygame.event.get():

            #Quit game
            if event.type == pygame.QUIT:
               running = False
               pygame.quit()

            #Directions keys
            if event.type == KEYDOWN:
               player.change_direct = 0 #If key is used, memory_direction has to update
               if event.key == K_UP:
                  player.direct = UP
               if event.key == K_DOWN:
                  player.direct = DOWN
               if event.key == K_LEFT:
                  player.direct = LEFT
               if event.key == K_RIGHT:
                  player.direct = RIGHT
               if event.key == K_ESCAPE:
                  pause_game()

         #Handles player movement
         player.time+=player.speed
         if player.time>=100:
            for qtd in range(player.time // 100):
               player.move()
            
            player.time=player.time%100
            #changes pato-man's mouth animation
            player.aberto=not player.aberto

         for i in range(4):
            phantom[i].time+=phantom[i].speed
            if phantom[i].time>=100:
               for j in range(phantom[i].time//100):
                  phantom[i].move(player.pos[1] // TS, player.pos[0] // TS, player.direct, 0, 0, 0)
               phantom[i].time=phantom[i].time%100
         #If after .move() memory_direction failed or not
         if player.change_direct == 2:
            player.change_direct = 1

         #pato-man eats a normal pellet
         if matriz[player.pos[1]][player.pos[0]] == 2:

            for coins in consumables: #Search the atual coins in coins_list
               if coins.rect.x == player.pos[0] and coins.rect.y == player.pos[1]:
                  coins.kill() #Remove from all groups
                  break
            matriz[player.pos[1]][player.pos[0]] = 0
            score.add(10)

            player.change_sound()

         #Pato-man eats a power pellet
         elif matriz[player.pos[1]][player.pos[0]] == 3:

            player.eaten_phantom=0#keeps track of currently eaten ghosts

            for i in range(4):
               if i==Inky or i==Clyde:
                  if (phantom[i].mode != EATEN):
                     if(phantom[i].mode == WAITING or phantom[i].mode == LEAVE):
                        phantom[i].frightened = True
                     else:
                        if(phantom[i].mode != FRIGHTENED):
                           phantom[i].change_direct()
                        phantom[i].mode = FRIGHTENED
                        phantom[i].reset_time()
               else:
                  #update ghots' state
                  if (phantom[i].mode != EATEN):
                     if(phantom[i].mode == LEAVE):
                        phantom[i].frightened = True
                     else:
                        if(phantom[i].mode != FRIGHTENED):
                           phantom[i].change_direct()
                        phantom[i].mode = FRIGHTENED
                     phantom[i].reset_time()
               
            for power in consumables: #Search the atual power in power_list
               if power.rect.x == player.pos[0] * TS and power.rect.y == player.pos[1] * TS:
                  power.kill() #Remove from all groups
                  break
            matriz[player.pos[1]][player.pos[0]] = 0
            score.add(50)

            player.change_sound()
               
         #music handler
         #------------------------------------------
         if (phantom[Blinky].mode == FRIGHTENED or phantom[Inky  ].mode == FRIGHTENED or phantom[Clyde ].mode == FRIGHTENED or phantom[Pinky ].mode == FRIGHTENED or phantom[Blinky].frightened or phantom[Inky  ].frightened or phantom[Clyde ].frightened or phantom[Pinky ].frightened):
            player.speed=player.fast_speed
            if now_music!='power.wav' and now_music!='eyes.wav':
               now_music='power.wav'
               play_music('power.wav',volume)

         if (phantom[Blinky].mode == EATEN or phantom[Inky  ].mode == EATEN or phantom[Clyde ].mode == EATEN or phantom[Pinky ].mode == EATEN):
            if now_music!='eyes.wav':
               now_music='eyes.wav'
               play_music('eyes.wav',volume)
         elif now_music=='eyes.wav' and not (phantom[Blinky].mode == EATEN or phantom[Inky  ].mode == EATEN or phantom[Clyde ].mode == EATEN or phantom[Pinky ].mode == EATEN):
            now_music=''

         if not (phantom[Blinky].mode == FRIGHTENED or phantom[Inky  ].mode == FRIGHTENED or phantom[Clyde ].mode == FRIGHTENED or phantom[Pinky ].mode == FRIGHTENED) and not (phantom[Blinky].mode == EATEN or phantom[Inky  ].mode == EATEN or phantom[Clyde ].mode == EATEN or phantom[Pinky ].mode == EATEN or phantom[Blinky].frightened or phantom[Inky  ].frightened or phantom[Clyde ].frightened or phantom[Pinky ].frightened):
            player.speed=player.normal_speed
            if now_music!=level_music:
               now_music=level_music
               play_music(level_music,volume)

         if len(consumables)==144:
            level_music = 'music2.wav'

         if len(consumables)==83:
            level_music = 'music3.wav'

         if len(consumables)==27:
            level_music = 'music4.wav'
         #------------------------------------------

         #goes back to menu when level is finished
         if len(consumables)==0:
            play_music('win_sound.wav',volume)

            for frame in range(10):
               if frame==5:
                  play_music('win_sound.wav',volume)
               
               screen.fill((0,0,0))
               score.display()
               lives.display()
               player.display()
               screen.blit(background_white, (0, 6 * 8))
               pygame.display.update()
               pygame.time.wait(100)

               screen.fill((0,0,0))
               score.display()
               lives.display()
               player.display()
               screen.blit(background_open, (0, 6 * 8))
               pygame.display.update()
               pygame.time.wait(100)
            nonlocal index
            index+=1
            if index==len(levels):
               win_game()
               pygame.time.wait(400)
               break
              
            game(current_lives = lives.lives,current_score=score.score,player_normal_speed=levels[index][0],player_fast_speed=levels[index][1],ghost_normal_speed=levels[index][2],ghost_slow_speed=levels[index][3],is_gamer=levels[index][4])
            break
         #Display objects on screen
         screen.fill((0, 0, 0))
         score.display()
         lives.display()
         screen.blit(background,(0, 6*8))  
         #Display coins and powers on screen
         consumables.draw(screen)
         for i in range(4):         
            screen.blit(phantom[i].img, (phantom[i].pos[0]-Ajuste, phantom[i].pos[1]-Ajuste))
         player.display()
         player.collision()
         pygame.display.update()


   index=0
   if is_gamer:
      levels=[[100,90,150,100,True],[100,90,150,100,True],[100,90,150,100,True],[100,90,150,100,True],[100,90,150,100,True],[100,90,150,100,True],[100,90,150,100,True],[100,90,150,100,True],[100,90,150,100,True]]
      game(current_lives=1,current_score='00',player_normal_speed=levels[index][0],player_fast_speed=levels[index][1],ghost_normal_speed=levels[index][2],ghost_slow_speed=levels[index][3],is_gamer=levels[index][4])
   else:
      levels=[[80,90,75,50,False],[90,95,85,55,False],[90,95,85,55,False],[90,95,85,55,False],[100,100,95,60,False],[100,100,95,60,False],[100,100,95,60,False],[100,100,95,60,False],[100,100,95,60,False],[100,100,95,60,False]]
      game(current_lives=3,current_score='00',player_normal_speed=levels[index][0],player_fast_speed=levels[index][1],ghost_normal_speed=levels[index][2],ghost_slow_speed=levels[index][3],is_gamer=levels[index][4])
   
   #victory screen   
   go_to_menu()
      
if __name__=='__main__':
   setup(0.10,True)
